Package: basictabler
Type: Package
Title: Construct Rich Tables for Output to 'HTML'/'Excel'
Version: 1.0.4
Authors@R: person("Christopher", "Bailiss", email = "cbailiss@gmail.com",
  role = c("aut", "cre"))
Description: Easily create tables from data 
    frames/matrices.  Create/manipulate tables 
    row-by-row, column-by-column or cell-by-cell.  
    Use common formatting/styling to output 
    rich tables as 'HTML', 'HTML widgets' or to 
    'Excel'. 
Depends: R (>= 3.3.0)
Imports: R6 (>= 2.2.0), dplyr (>= 0.5.0), htmltools(>= 0.3.5),
        htmlwidgets (>= 0.8)
Suggests: jsonlite (>= 1.1), lubridate (>= 1.5.0), listviewer (>=
        1.4.0), openxlsx (>= 4.0.17), flextable (>= 0.6.6), officer (>=
        0.3.18), shiny, knitr, rmarkdown, testthat
License: GPL-3
Encoding: UTF-8
LazyData: true
URL: http://www.basictabler.org.uk/,
        https://github.com/cbailiss/basictabler
BugReports: https://github.com/cbailiss/basictabler/issues
VignetteBuilder: knitr
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2025-04-21 20:59:59 UTC; Chris
Author: Christopher Bailiss [aut, cre]
Maintainer: Christopher Bailiss <cbailiss@gmail.com>
Repository: CRAN
Date/Publication: 2025-04-21 21:40:02 UTC
Built: R 4.3.3; ; 2025-04-21 23:49:36 UTC; unix
