# Generated by roxygen2: do not edit by hand

export(BasicTable)
export(TableCell)
export(TableStyle)
export(TableStyles)
export(basictabler)
export(basictablerOutput)
export(basictablerSample)
export(getSimpleColoredTblTheme)
export(getTblTheme)
export(qhtbl)
export(qtbl)
export(renderBasictabler)
import(dplyr)
import(htmltools)
import(htmlwidgets)
importFrom(R6,R6Class)
